function [NRMSE NRMSE_FCLS RE SAM RE_VCA SAM_VCA RE_Est SAM_Est Label_est] =  exploitation...
    (Y,Y_bloc,TalphaPlus,alpha0,alpha_FCLS,TMPlus,MPlus,MEEA,Tsigma2,sigma2,TLabel,label0 ,Taux_acceptC,Taux_acceptM,Taux_acceptsig,K,Nbi,Nmc,display);

%------------------------------------------------------------------
% This function allows to exploit the results provided by
%       the function unmixing
%
% INPUT
%       Y              : Pixels NxNxL
%       Y_bloc         : Pixels LxN2
%       TalphaPlus     : Abundance chains
%       alpha0         : Actual abundances (equal to FCLS for real data)
%       alpha_FCLS     : FCLS abundances
%       TMPlus         : Endmember mean chains
%       MPlus          : Actual endmembers (equal to MEEA for real data)
%       MEEA           : Extracted M using an EEA
%       Tsigma2        : Endmember variance chains
%       sigma2         : Actual endmember variances
%       TLabel         : Label chains
%       label0         : Actual labels
%       Taux_acceptC   : Acceptance rate for abundances
%       Taux_acceptM   : Acceptance rate for endmember means
%       Taux_acceptsig : Acceptance rate for endmember variances
%       K              : Number of spatial classes
%       Nbi            : Burn-in number
%       Nmc            : Number of samples
%       display        : 0 or 1
%
%
% OUTPUT
%       NRMSE          : abundance RMSE (compared to FCLS for real data)
%       NRMSE_FCLS     : FCLS abundance RMSE
%       RE             : Reconstruction error (yobs,yrec)
%       SAM            : Spectral angle mapper (yobs,yrec)
%       RE_VCA         : VCA reconstruction error (yobs,yrec)
%       SAM_VCA        : VCA spectral angle mapper (yobs,yrec)
%       RE_Est         : Endmember reconstruction error (MPlus,Mest)
%       SAM_Est        : Spectral angle mapper(MPlus,Mest)
%
%------------------------------------------------------------------

N  = sqrt(size(TalphaPlus,2));
Nr = Nmc - Nbi;
Nc = floor(sqrt(Nmc-Nbi+1));
R  = size(TalphaPlus,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%% Organisation of the spectra
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Nintervalle = 6;
divsp =  floor(linspace(1,Nr,Nintervalle));
for r=1:R
    Mest(:,r) = mean(squeeze(TMPlus(:,r,1:Nr)),2);
    
    for k=1:Nintervalle-2
        Mestk(:,r,k) = mean(squeeze(TMPlus(:,r,divsp(k):divsp(k+1))),2);
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
pos                 = 1:R;
pos3                = [];

for i =1:R
    dist(i,:)    = (sum((repmat(Mest(:,i),1,R) - MPlus(:,1:R)).^2));
    [v pos3(i)]   = min(dist(i,:));
end
pos3
MPlus       = MPlus(:,pos3);
MEEA        = MEEA(:,pos3);
alpha0      = alpha0(pos3,:);
alpha_FCLS  = alpha_FCLS(pos3,:);
sigma2      = sigma2(:,pos3);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Pixel 1: Abundances
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
i=20;
if(display==1)
figure
end
TalphaPlus2 = squeeze(TalphaPlus(:,i,:));
alpha02     = alpha0(:,i);
for i=1:R
    vect    = NaN*zeros(1,Nc+1);
    Talphai = TalphaPlus2(i,1:Nr);
    xmin = min(Talphai)*0.8;
    xmax = max(Talphai)*1.2;
    
    pas  = (xmax-xmin)/Nc;
    hi   = hist(Talphai,xmin:pas:xmax);
    hi   = hi/(pas*Nr);
    [val pos] = (min(abs((xmin:pas:xmax)-alpha02(i))));
    vect(pos) = 1;
    if(display==1)
        subplot(1,R,i)
        plot(xmin:pas:xmax,hi);hold on;stem(xmin:pas:xmax,vect*max(hi),'r')
        set(gca,'ytick',[]);
        axis([xmin xmax 0 max(hi)*1.1]);
        ylabel(['Posterior distribution of \alpha_' int2str(i)]);
    end
end
result = 1;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Image: Abundances
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for j = 1:N
    for i = 1:N
        n                   = i+(j-1)*N;
        Talpha(i,j,:)       = mean(TalphaPlus(:,n,1:Nr),3);
        Talpha2(:,n)        = mean(TalphaPlus(:,n,1:Nr),3);
        TalphaExact(i,j,:)  = (alpha0(:,n));
    end
end
if(display==1)
    for r=1:R
        figure(r+1); subplot(1,3,1);imagesc(TalphaExact(:,:,r));caxis([0 1])
        subplot(1,3,2);imagesc(Talpha(:,:,r));caxis([0 1])
        subplot(1,3,3);imagesc(abs(TalphaExact(:,:,r)-Talpha(:,:,r)));caxis([0 0.1])
    end
end

%%%% NRMSE abundances
NRMSE      = sqrt(1/N^2/R*norm(Talpha2 - alpha0)^2);
NRMSE_FCLS = sqrt(1/N^2/R*norm(alpha_FCLS - alpha0)^2);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%     Projected data      %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Cov_data   = cov((Y_bloc'));  % L X N2    N2 X L
[V,D]      = eig(Cov_data);
Vproj      = V(:,end-(R-2):end);

Pix_proj    = Vproj'*Y_bloc; % 2 X N2
EM_proj     = Vproj'*MPlus; % 2 X R
EEEA_proj   = Vproj'*MEEA; % 2 X R
Mest_proj   = Vproj'*Mest; % 2 X R


for k=1:Nintervalle-2
    Mest_projk(:,1,k) = Vproj'*squeeze(Mestk(:,1,k));
    Mest_projk(:,2,k) = Vproj'*squeeze(Mestk(:,2,k));
    Mest_projk(:,3,k) = Vproj'*squeeze(Mestk(:,3,k));
end
if(display==1)
    figure;plot(Pix_proj(1,:),Pix_proj(2,:),'+');
    hold on; plot(EM_proj(1,:),EM_proj(2,:),'*r','Markersize',10)
    hold on; plot(EEEA_proj(1,:),EEEA_proj(2,:),'dg','Markersize',10)
    hold on; plot(Mest_proj(1,:),Mest_proj(2,:),'vk','Markersize',10)
    for k=1:Nintervalle-2
        hold on; plot(squeeze(Mest_projk(1,:,k)),squeeze(Mest_projk(2,:,k)),'vk','Markersize',10)
    end
end
% figure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%     Reconstruction error     %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for i = 1:N
    for j = 1:N
        n               = i+(j-1)*N;
        Y_est(i,j,:)    = Mest*Talpha2(:,n);
        REn(n)          = norm(squeeze(Y(i,j,:)-Y_est(i,j,:)))^2;
        SAMn(n)         = acos((squeeze(Y(i,j,:))')*squeeze(Y_est(i,j,:))/norm(squeeze(Y(i,j,:)))/norm(squeeze(Y_est(i,j,:))) );
        
        for r=1:R
            REn_VCA(n,r)          = norm(squeeze(MPlus(:,r)-MEEA(:,r)))^2;
            SAMn_VCA(n,r)         = acos(MPlus(:,r)'*MEEA(:,r)/norm(MPlus(:,r))/norm(MEEA(:,r)) );
            REn_Est(n,r)          = norm(squeeze(MPlus(:,r)-Mest(:,r)))^2;
            SAMn_Est(n,r)         = acos(MPlus(:,r)'*Mest(:,r)/norm(MPlus(:,r))/norm(Mest(:,r)) );
        end
        
    end
end


RE  = sqrt(1/size(Y,3)/N^2*sum(REn));
SAM = mean(SAMn);

RE_VCA = sqrt(1/size(Y,3)/N^2*sum(REn_VCA,1));
SAM_VCA = mean(SAMn_VCA);

RE_Est = sqrt(1/size(Y,3)/N^2*sum(REn_Est,1));
SAM_Est = mean(SAMn_Est);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%  Points in the simplex  %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(display==1)
    figure
    plot(Talpha2(1,1:N^2),Talpha2(2,1:N^2),'b+')
    hold on;plot(alpha0(1,1:N^2),alpha0(2,1:N^2),'r*')
    
    
    
    figure;subplot(2,1,1);
    plot(mean(Taux_acceptC));title('Tc de \alpha')
    subplot(2,1,2);plot(mean(Taux_acceptM));title('Tc de M')
    
    
    
    figure;plot(squeeze(sigma2(1,1,:)));title('Chaine de \sigma');xlabel('Iterations')
    
    figure;plot(squeeze(TMPlus(1:20:end,1,:))');hold on;plot(repmat(MPlus(1:20:end,1)',Nr,1),'--')
    title('Chains of M');xlabel('Iterations')
    
    figure;plot(squeeze(TalphaPlus(1,1:100:end,:))');hold on;plot(repmat(alpha0(1,1:100:end),Nr,1),'--')
    title('Chains of alpha');xlabel('Iterations')
    
    figure;hSLines=plot(MPlus);hold on;hCLines=plot(Mest,'*');hold on;hFLines=plot(MEEA,'--');
    title('Endmembers');
    hSGroup = hggroup;
    hCGroup = hggroup;
    hFGroup = hggroup;
    set(hSLines,'Parent',hSGroup);
    set(hCLines,'Parent',hCGroup);
    set(hFLines,'Parent',hFGroup);
    % Include these hggroups in the legend:
    set(get(get(hSGroup,'Annotation'),'LegendInformation'),...
        'IconDisplayStyle','on');
    set(get(get(hCGroup,'Annotation'),'LegendInformation'),...
        'IconDisplayStyle','on');
    set(get(get(hFGroup,'Annotation'),'LegendInformation'),...
        'IconDisplayStyle','on');
    legend('Vrai','ULMM','VCA')
end

i = 1 ;j = 1; coeff_var = 3;
n               = i+(j-1)*N;
% sigma  =  sqrt(mean(sigma2(i,j,1:Nr),3));

if(display==1)
    figure
    hold off
    subplot(3,1,1)
    ylabel('Pixel 1')
    plot(Mest(:,1),'LineWidth',2);hold on
    plot(MPlus(:,1) ,'r--','LineWidth',2)
    plot(MEEA(:,1) ,'g--','LineWidth',2)
    plot(Mest(:,1)+coeff_var*sqrt(squeeze(mean(Tsigma2(1,:,1:Nr),3))') ,'b--','LineWidth',2)
    plot(Mest(:,1)-coeff_var*sqrt(squeeze(mean(Tsigma2(1,:,1:Nr),3))') ,'b--','LineWidth',2)
    
    axis([0 207 0 1.1])
    ylabel('m_1')
    subplot(3,1,2)
    plot(Mest(:,2),'LineWidth',2);hold on
    plot(MPlus(:,2) ,'r--','LineWidth',2)
    plot(MEEA(:,2) ,'g--','LineWidth',2)
    plot(Mest(:,2)+coeff_var*sqrt(squeeze(mean(Tsigma2(2,:,1:Nr),3))') ,'b--','LineWidth',2)
    plot(Mest(:,2)-coeff_var*sqrt(squeeze(mean(Tsigma2(2,:,1:Nr),3))') ,'b--','LineWidth',2)
    axis([0 207 0 1.1])
    ylabel('m_2')
    subplot(3,1,3)
    plot(Mest(:,3),'LineWidth',2);hold on
    plot(MPlus(:,3) ,'r--','LineWidth',2)
    plot(MEEA(:,3) ,'g--','LineWidth',2)
    plot(Mest(:,3)+coeff_var*sqrt(squeeze(mean(Tsigma2(3,:,1:Nr),3))') ,'b--','LineWidth',2)
    plot(Mest(:,3)-coeff_var*sqrt(squeeze(mean(Tsigma2(3,:,1:Nr),3))') ,'b--','LineWidth',2)
    xlabel('Bands')
    ylabel('m_3')
    axis([0 207 0 1.1])
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% Analysis of sigma2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(display==1)
    figure;plot(squeeze(mean(Tsigma2(:,:,1:Nr),3))')
    hold on;plot(sigma2,'--');title('variation en fct de L')
    xlabel('bandes')
    
    
    figure;plot(mean(Taux_acceptsig));title('Tc \sigma^2')
    figure;plot(squeeze(Tsigma2(:,100,:))'); title('Chains \sigma^2')
    hold on;plot(repmat(sigma2(100,:),Nr,1),'--');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% Analysis of labels
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[vallab posLab] = max(hist(squeeze(TLabel(1,:,1:Nr))',0:K));Label_est  = posLab-1;

Cov_data   = cov((Y_bloc'));  % L X N2    N2 X L
[V,D]      = eig(Cov_data);
Vproj      = V(:,end-(R-2):end);
Pix_proj    = Vproj'*Y_bloc; % 2 X N2
EM_proj     = Vproj'*MPlus; % 2 X R
EEEA_proj   = Vproj'*MEEA; % 2 X R
if(display==1)
    figure;plot(Pix_proj(1,:),Pix_proj(2,:),'+');
    hold on; plot(EM_proj(1,:),EM_proj(2,:),'*r','Markersize',10)
    hold on; plot(EEEA_proj(1,:),EEEA_proj(2,:),'dg','Markersize',10)
    pos1=find(Label_est==1);
    plot(Pix_proj(1,pos1),Pix_proj(2,pos1),'r+');
    pos2=find(Label_est==2);
    plot(Pix_proj(1,pos2),Pix_proj(2,pos2),'g+');
    pos3=find(Label_est==3);
    plot(Pix_proj(1,pos3),Pix_proj(2,pos3),'k+');
    
    figure;imagesc(reshape(Label_est,N,N));colormap('gray')
end
for k=1:K
    indk     = find(label0==k);
    for j=1:K
        Conf_tab(k,j) = sum(Label_est(indk)==j);
    end
end

Conf_tab



